# -*- coding: iso-8859-1 -*-

# renamePageWithAttributes.py --- Plugin XML-RPC command for MoinMoin renaming a page

# $Id: renamePageWithAttributes.py,v 1.3 2006/08/09 20:40:14 stefan Exp $

# Copyright 2006 Stefan Merten <smerten@oekonux.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import xmlrpclib

from MoinMoin import wikiutil, version
from MoinMoin.PageEditor import PageEditor
from MoinMoin.action.RenamePage import RenamePage

###############################################################################
###############################################################################

class RenamePageXmlRpc(RenamePage):
    """
    A wrapper around the original action smoothing away some ugly things.

    Used only for MoinMoin versions 1.3.x and earlier.

    Note: This is an old style class so C{super} can not be used.
    """

    ###########################################################################

    def __init__(self, fromname, toname, request):
        RenamePage.__init__(self, fromname, request)
        self.toname = toname

    ###########################################################################

    def allowed(self):
        # `RenamePage.allowed()' does not check the target page and uses the
        # class name for checking the operation to be allowed - unusable here
        return ('RenamePage' in self.request.cfg.allowed_actions
                and self.request.user.may.write(self.toname)
                and self.request.user.may.delete(self.pagename))

    ###########################################################################

    def pageExistsError(self, pagename):
        self.error = pagename

    ###########################################################################

    def rename(self, comment):
        # Fill out the values of a form since `RenamePage.rename()' uses these
        # directly; yes, this is an ugly hack...
        self.request.form = { 'ticket': ( wikiutil.createTicket(), ),
                              'comment': ( comment, ),
                              'newpagename': ( self.toname, ),
                              }
        RenamePage.rename(self)

###############################################################################
###############################################################################

def execute(xmlRpc, fromname, toname, attributes):
    """
    Callback to execute the functionality of this plugin.

    Rename a page using all the security mechanisms described for C{putPage}.
    In particular the configuration options C{xmlrpc_putpage_enabled} and
    C{xmlrpc_putpage_trusted_only} are considered.

    @param xmlRpc: Describes the local execution environment. While all other
                   parameters come from the remote caller this one is added by
                   the local execution environment.
    @type xmlRpc: XmlRpcBase

    @param fromname: The original name of the page to rename.
    @type fromname: unicode | str

    @param toname: The new name of the page to rename.
    @type toname: unicode | str

    @param attributes: Attributes describing the action further. See
                       http://www.merten-home.de/FreeSoftware/remoteMoin/manual.html#attributes
                       for the rationale.
    @type attributes: { 'comment': str | unicode, ..., }

    @return: Flag whether operation was successful or fault.
    @rtype: xmlrpclib.Boolean | xmlrpclib.Fault
    """
    # This is taken from `wikirpc.XmlRpcBase.xmlrpc_putPage` and modified. See
    # original `xmlrpc_putPage` comments for the configuration stuff.

    fromname = xmlRpc._instr(fromname)
    toname = xmlRpc._instr(toname)
    comment = xmlRpc._instr(attributes.get('comment', u''))
    permissionDenied = xmlrpclib.Fault(1, "You are not allowed to rename this page")
    if not xmlRpc.request.cfg.xmlrpc_putpage_enabled:
        return permissionDenied
    if (xmlRpc.request.cfg.xmlrpc_putpage_trusted_only
        and not xmlRpc.request.user.trusted):
        return permissionDenied

    versionNumbers = version.release.split('.')
    if int(versionNumbers[0]) <= 1 and int(versionNumbers[1]) <= 3:
        # MoinMoin 1.3.4 has only an action implementing this
        renamePage = RenamePageXmlRpc(fromname, toname, xmlRpc.request)
        if not renamePage.allowed():
            return permissionDenied
        if not renamePage.page.exists():
            return xmlRpc.noSuchPageFault()

        renamePage.rename(comment)

        if renamePage.error is None:
            return xmlrpclib.Boolean(1)
        elif renamePage.error == toname:
            return xmlrpclib.Fault(1, "Page already exists")
        else:
            return xmlrpclib.Fault(1, self.error)
    else:
        # MoinMoin 1.5.4 has a method in `PageEditor`
        if (not xmlRpc.request.user.may.write(toname)
            or not xmlRpc.request.user.may.write(fromname)
            or not xmlRpc.request.user.may.delete(fromname)):
            return permissionDenied
        fromPage = PageEditor(xmlRpc.request, fromname)
        if not fromPage.exists():
            return xmlRpc.noSuchPageFault()
        if not toname:
            # This is not a particular good error for an empty target name...
            return xmlRpc.noSuchPageFault()

        ( success, dummy, ) = fromPage.renamePage(toname, comment)
        if success:
            return xmlrpclib.Boolean(1)
        else:
            # Assumes that in practice all errors returned are of this type
            return xmlrpclib.Fault(1, "Page already exists")
